﻿using Avalonia.Controls;
using AvaloniaBaseTeacher.Models;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Storage.ValueConversion;
using MsBox.Avalonia;
using MsBox.Avalonia.Enums;
using ReactiveUI;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AvaloniaBaseTeacher.ViewModels
{
    internal class show : ViewModelBase
    {
        
        List<Teacher> _teachList;

        public List<Teacher> teachList
        {
            get => _teachList;
            set => this.RaiseAndSetIfChanged(ref _teachList, value);
        }

        public show()
        {
            teachList = MainWindowViewModel.myConnection.Teachers.Include(x => x.GenderNavigation).Include(x => x.Course1Teachers).ThenInclude(x => x.IdCourseNavigation).Include(x => x.Discipline1Teachers).ThenInclude(x => x.IdDisciplineNavigation).ToList();
        }

        public void AddTeacher()
        {
            MainWindowViewModel.Instance.PageContent = new Add();
        }

        public async void DeleteTeacher(int idTeacher)
        {
            ButtonResult result = await MessageBoxManager.GetMessageBoxStandard("Подтверждение выбора", "Вы точно хотите удалить пользователя?", ButtonEnum.YesNo).ShowAsync();
            if (result == ButtonResult.Yes)
            {
                Teacher deleteTeacher = MainWindowViewModel.myConnection.Teachers.First(x => x.IdTeacher == idTeacher);
                MainWindowViewModel.myConnection.Teachers.Remove(deleteTeacher);
                MainWindowViewModel.myConnection.SaveChanges();
                MainWindowViewModel.Instance.PageContent = new Show();
                ButtonResult result1 = await MessageBoxManager.GetMessageBoxStandard("Успешное удаление", "Пользователь удалён", ButtonEnum.Ok).ShowAsync();
            }
            else { }
        }

        public void UpdateTeacher(int idTeacher)
        {
            MainWindowViewModel.Instance.PageContent = new Add(idTeacher);
        }

        string _searchTeacher;    
        public string SearchTeacher
        {
            get => _searchTeacher ;
            set
            {
                _searchTeacher = value;
                AllFilters();
            }
        }

        public bool _gender = false;
        public bool _gender1 = false;
        public bool Gender
        {
            get=>_gender;
            set
            {
                _gender = value;
                AllFilters();
            }
        }

        public bool Gender1
        {
            get => _gender1;
            set
            {
                _gender1 = value;
                AllFilters();
            }
        }

        public bool _phone = false;
        public bool Phone
        {
            get => _phone;
            set
            {
                _phone = value;
                AllFilters();
            }
        }

        List<Discipline> _disciplinesfilter = [new Discipline() { IdDiscipline = 0, NameDiscipline = "Все дисциплины" }, .. MainWindowViewModel.myConnection.Disciplines.ToList()];
        public List<Discipline> DisciplinesFilter { get => _disciplinesfilter; }
        Discipline _selecteddiscipline = null;
        public Discipline SelectedDiscipline
        {
            get
            {
                if (_selecteddiscipline == null) return _disciplinesfilter[0];
                else { return _selecteddiscipline; }
            }
            set
            {
                _selecteddiscipline = value;
                AllFilters();
            }
        }

        int _selectedSort = 0;
        bool _Visible = false;
        bool _Up = true;
        bool _Down = true;

        public int SelectedSort { get => _selectedSort; set { _selectedSort = value; AllFilters(); }  }
        public bool Visible { get => _Visible; set => this.RaiseAndSetIfChanged(ref _Visible, value); }
        public bool Up { get => _Up; set { this.RaiseAndSetIfChanged(ref _Up, value);AllFilters(); } }
        public bool Down { get => _Down; set { this.RaiseAndSetIfChanged(ref _Down, value); AllFilters(); } }

       
        void AllFilters()
        {
            teachList = MainWindowViewModel.myConnection.Teachers.Include(x => x.GenderNavigation).Include(x => x.Course1Teachers).ThenInclude(x => x.IdCourseNavigation).Include(x => x.Discipline1Teachers).ThenInclude(x => x.IdDisciplineNavigation).ToList();
            if (!string.IsNullOrWhiteSpace(_searchTeacher))
            {
                teachList = teachList.Where(x => x.FIO.ToLower().Contains(_searchTeacher.ToLower())).ToList();
            }

            if (_gender || _gender1)
            {
                teachList = teachList.Where(x => (_gender && x.Gender == 1) || (_gender1 && x.Gender == 2)).ToList();
            }

            if (_phone)
            {
                teachList = teachList.Where(x=>x.Phone!="").ToList();
            }

            if(_selecteddiscipline != null && _selecteddiscipline.IdDiscipline!=0)
            {
                teachList = teachList.Where(x=>x.Discipline1Teachers.Any(y=>y.IdDisciplineNavigation.IdDiscipline == _selecteddiscipline.IdDiscipline)).ToList();
            }

            switch(_selectedSort)
            {
                case 0:
                    Visible = false;
                    break;
                case 1:
                    Visible = true;
                    if(_Down)
                    {
                        teachList = teachList.OrderByDescending(x=>x.Experience).ToList();
                    }
                    else
                    {
                        teachList = teachList.OrderBy(x => x.Experience).ToList();
                    }
                    break;  
                case 2:
                    Visible = true;
                    if (_Down)
                    {
                        teachList = teachList.OrderByDescending(x => x.Surname).ToList();
                    }
                    else
                    {
                        teachList = teachList.OrderBy(x => x.Surname).ToList();
                    }
                    break;
                case 3:
                    Visible = true;
                    if (_Down)
                    {
                        teachList = teachList.OrderByDescending(x=>x.hour).ToList();
                    }
                    else
                    {
                        teachList = teachList.OrderBy(x => x.hour).ToList();
                    }
                    break;

            }
        }
    }
  
}
